source("~/Documents/share/R/Disp_an/GeneralTr/funcsAR_4.R")
#source("~/Documents/share/R/Disp_an/GeneralTr/EMPA.R")
library(MASS)
library(pracma)
# 
# моделирование данных для дисперсионного анализа повторных наблюдений

Model.ARM<-function(alpha,beta,sigma2,sigma2.1,n.a)
{
  I_<-length(alpha)
  T_<-length(beta)
  gamma<-matrix(rnorm(I_*T_,0,0.00001),ncol=T_);gamma
  n<-sum(n.a)
  Gamma_<-gamma
  Gamma_<-sapply(seq(T_),function(t)Gamma_[,t]+beta[t])
  Gamma_<-t(sapply(seq(I_),function(i)Gamma_[i,]+alpha[i]))
  Gamma_
  
  A<-Reduce('rbind',
         lapply(seq(I_),function(i)matrix(rep(Gamma_[i,],n.a[i]),ncol=T_,byrow=TRUE)))
  A1<-apply(A,2,function(x)x+rnorm(n,0,sqrt(sigma2)))
  group<-do.call('c',apply(cbind(n.a,c(1,2)),1,function(x)rep(x[2],x[1])))
  data.frame(group=group,t(apply(A1,1,function(x)x+rnorm(1,0,sqrt(sigma2.1)))))
}

# моделирование данных для дисперсионного анализа повторных наблюдений со случайными эффектами

Model.ARM.R<-function(alpha,sigma2,sigma2.1,s.g,s.b,T_,n.a)
{
  I_<-length(alpha)
  
  gamma<-matrix(rnorm(I_*T_,0,sqrt(s.g)),ncol=T_);gamma
  gamma<-matrix(rep(0,T_*I_),ncol=T_);gamma
  
  n<-sum(n.a)
  Gamma_<-gamma
  Gamma_<-sapply(seq(T_),function(t)Gamma_[,t]+rnorm(1,0,sqrt(s.b)))
  Gamma_<-t(sapply(seq(I_),function(i)Gamma_[i,]+alpha[i]))
  Gamma_
  
  A<-Reduce('rbind',
            lapply(seq(I_),function(i)matrix(rep(Gamma_[i,],n.a[i]),ncol=T_,byrow=TRUE)))
  A1<-apply(A,2,function(x)x+rnorm(n,0,sqrt(sigma2)))
  group<-do.call('c',apply(cbind(n.a,c(1,2)),1,function(x)rep(x[2],x[1])))
  data.frame(group=group,t(apply(A1,1,function(x)x+rnorm(1,0,sqrt(sigma2.1)))))
}
#-----------------
# пример
alpha<-c(1,2);beta<-c(0,0,0); n.a<-c(15,14);
sigma2<-1;sigma2.1<-4
r<-length(alpha);n<-sum(n.a);T_<-length(beta)
data<-Model.ARM(alpha,beta,sigma2,sigma2.1,n.a)
Group<-TRUE
eps<-1/10^5

# matrices <- matrices.list(as.matrix(data), data)
# B<-Bias(matrices,Group)
# ii<-c(1,1)
# MC<-Mick.Computation(c(t=1,tau=2),B,ii=ii,Group);
# 
# Cov.F(ii,B,Group)
# CV<-Covariance.matrix(B,Group)
# 
 p.values.AU<-AU.2(data,eps,Group)$P.values



#-------------------------------------
AR.One<-function(dat.AR)
{
  m<-ncol(dat.AR)-1;m
  dat.AR<-na.omit(dat.AR)
  dat.AR.T<-data.frame( stack(dat.AR[,-1]),
                        sub=as.factor(rep(seq(nrow(dat.AR)),m)), 
                        gr=as.factor(rep(dat.AR$group,m)))
  dat.AR.T$values<-as.numeric(dat.AR.T$values)
  
  formula<-values~gr*ind+Error(sub/ind)
  
  aov.out <- aov(formula, data=dat.AR.T)
  aov.S<-summary(aov.out)
  
  pp<-c(pp1<-aov.S[[1]][[1]][1,5],aov.S[[2]][[1]][seq(2),5])
  
  wm<-with(dat.AR.T, tapply(values, list(gr,ind), mean))
  wsd<-with(dat.AR.T, tapply(values, list(gr,ind), sd))
  wn<-with(dat.AR.T, tapply(values, list(gr,ind), length))
  
  Means<-list(wm=wm,wsd=wsd,wn=wn)
  
  names(pp)<-c("p.gr","p.time","p.gr.time")
  list(pp=pp,Means=Means)
}
#----------
#-----------Сравнение со стандартной процедурой в R
cbind(AR.One(data)[[1]],p.values.AU[-1])

#----------------------------
#-----моделируем пропуски
Mod.missing<-function(data){
  data[c(1),3]<-NA
  data[c(2),4]<-NA
  data[c(3),4]<-NA
  data[nrow(data),3]<-NA
  data[nrow(data),4]<-NA
  data[nrow(data)-1,4]<-NA
  data[nrow(data)-3,3]<-NA
  data[nrow(data)-4,3]<-NA
  data
}
 
 
 data<-Model.ARM(alpha=c(1,-1),beta=c(0,0,0),sigma2,sigma2.1,n.a=c(10,20))
 data<-Mod.missing(data)
 au<-AU.2(data,eps,Group)
 p.values.AU<-au$P.values  
 p.values.AU

 Mt<-lapply(au$ep$B$First,function(x)x$Counts$Mt);Mt
 
#-------------------
# Проверка равномерного распределения для критериев
# F_b


pp_b<-sapply(seq(150),function(x)
  AU.2(Mod.missing(Model.ARM(alpha=c(1,-1),beta=c(0,0,0),sigma2,sigma2.1,n.a=c(10,20))),
       eps,Group=TRUE)$P.values[3])
plot(ecdf(c(pp_b)))
lines(c(0,1),c(0,1),type="l",col=2)

title(sub=paste("p_b",format(ks.test(pp_b, "punif")$p.value,3,2),sep="="))





# F_g
pp_g<-sapply(seq(150),function(x)
  AU.2(Mod.missing(Model.ARM(alpha=c(-1,1),beta=c(0,0,0),sigma2,sigma2.1,n.a=c(10,20))),
       eps,TRUE)$P.values[4])

plot(ecdf(pp_g))
lines(c(0,1),c(0,1),type="l",col=2)
title(sub=paste("p_g",format(ks.test(pp_g, "punif")$p.value,3,2),sep="="))





# F_a
pp_a<-sapply(seq(150),function(x)
  AU.2(Mod.missing(Model.ARM(alpha=c(0,0),beta=c(1,2,3),
                 sigma2,sigma2.1,n.a=c(10,20))),eps,Group)$P.values[2])

plot(ecdf(pp_a))
lines(c(0,1),c(0,1),type="l",col=2)
title(sub=paste("F_a",format(ks.test(pp_a, "punif")$p.value,3,2),sep="=" ))



